package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionPartContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionPsOutContext;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description : 保存加工单信息工步
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2024/6/11 18:50
 * @Modify:
 **/
@Slf4j
@Service("mesWorkOrderSaveStepService")
public class MesWorkOrderSaveStepService extends BaseStepService {
    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private IMesProductionDispatchContextStepService productionDispatchContextStepService;

    @Override
    public StepResult execute(StationRequestBean reqBean) {
        StationResultBean resultBean = new StationResultBean();
        StepResult stepResult = StepResult.getSuccessComplete();
        //1. 获取上下文中的MesProductionPartContext 产出零件
        List<MesProductionPartContext> productionPartContext = productionDispatchContextStepService.getProductionPartContext(reqBean);
        if (productionPartContext.isEmpty()){
            return execSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, "没有可保存的加工单信息");
        }
        // 判断为空腔 isFinishedCode = true 就是空腔
        // stream 过滤 isFinishedCode = false  并且 foreignKey != null  并且 加工单code 为不为空;
        List<MesProductionPartContext> filteredPartContext = productionPartContext.stream().filter(item -> item.getIsFinishCode() == CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue() && item.getForeignKey() != null && item.getWorkOrderNo() != null).collect(Collectors.toList());
        if (filteredPartContext.isEmpty()){
            return execSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, "过滤后,没有可保存的加工单信息");
        }
        // 怎么判断条码生成过了？ 看产出零件和入料，in 要判端relateId有值也需要新增, out生成条码 上下文
        // 如果没有in 需要加，
        // 如果有out条码,则判断id是否为空，不为空则是新生成的

        //2. 判断是否为空,id是否为空，如果没有则返回,id为工单id
        List<MesProductionPartContext> collectContext = productionPartContext.stream().filter(item -> item.getId() != null).collect(Collectors.toList());
        //3. 如果id不为空，查询工单信息
        if (collectContext.isEmpty()){
            return execSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, "没有可保存的加工单信息");
        }
        //4. 判断当前产出的零件MesProductionPsOutContext中是否有id，如果有就是新生成的
        List<MesProductionPsOutContext> productionPsOutContext = productionDispatchContextStepService.getProductionPsOutContext(reqBean);
        //过滤出新生成的条码，没有条码id就是新生成的
        List<MesProductionPsOutContext> produceSnList = productionPsOutContext.stream().filter(item -> item.getId() != null).collect(Collectors.toList());
        if (produceSnList.isEmpty()){
            return execSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, "没有可保存的加工单信息");
        }
//        productionPsOutContext.stream().collect(Collectors.groupingBy(MesProductionPsOutContext::getId))
        //5. 修改完成数量，完成数量大于计划数量，就需要修改工单的状态为完成
        return execSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, "保存加工单信息成功");
    }
}
